--[[
RegisterSpecialization

Author:		Ifko[nator]
Date:		18.11.2021
Version:	2.0

History:	v1.0 @02.01.2019 - initial implementation in FS 19
			---------------------------------------------------
			v2.0 @18.11.2021 - convert to FS 22
			v2.1 @19.12.2022 - bugfix in call to "addSpecialization" (line 80). removed excess attribute
]]

RegisterSpecialization = {};
RegisterSpecialization.currentModDirectory = g_currentModDirectory;

local modDesc = loadXMLFile("modDesc", RegisterSpecialization.currentModDirectory .. "modDesc.xml");

RegisterSpecialization.debugPriority = Utils.getNoNil(getXMLInt(modDesc, "modDesc.registerSpecializations#debugPriority"), 0);

local function printError(errorMessage, isWarning, isInfo)
	local prefix = "::ERROR:: ";
	
	if isWarning then
		prefix = "::WARNING:: ";
	elseif isInfo then
		prefix = "::INFO:: ";
	end;
	
	print(prefix .. "from the RegisterSpecialization.lua: " .. tostring(errorMessage));
end;

local function printDebug(debugMessage, priority, addString)
	if RegisterSpecialization.debugPriority >= priority then
		local prefix = "";
		
		if addString then
			prefix = "::DEBUG:: from the RegisterSpecialization.lua: ";
		end;
		
		print(prefix .. tostring(debugMessage));
	end;
end;

local specializationNumber = 0;

while true do
	local specializationKey = string.format("modDesc.registerSpecializations.registerSpecialization(%d)", specializationNumber);
	
	if not hasXMLProperty(modDesc, specializationKey) then
		break;
	end;
	
	local specializationName = Utils.getNoNil(getXMLString(modDesc, specializationKey .. "#name"), "");
	local specializationClassName = Utils.getNoNil(getXMLString(modDesc, specializationKey .. "#className"), "");
	local specializationFilename = Utils.getNoNil(Utils.getFilename(getXMLString(modDesc, specializationKey .. "#filename"), RegisterSpecialization.currentModDirectory), ""); 
	local searchedSpecializations = string.split(Utils.getNoNil(getXMLString(modDesc, specializationKey .. "#searchedSpecializations"), ""), " ");
	
	local searchedSpecializationsString = "";
	
	local function getSearchedSpecializations(searchedSpecializations)
		for _, searchedSpecialization in pairs(searchedSpecializations) do
			if searchedSpecializationsString ~= "" then 
				searchedSpecialization = ", " .. searchedSpecialization; 
			end;
			
			searchedSpecializationsString = searchedSpecializationsString .. searchedSpecialization;
		end;
		
		return searchedSpecializationsString;
	end;
	
	printDebug("specializationName =  " .. specializationName .. " specializationClassName " .. specializationClassName .. " specializationFilename = " .. specializationFilename .. " searchedSpecializations = " .. getSearchedSpecializations(searchedSpecializations), 1, true);
	
	if specializationName ~= ""
		and specializationClassName ~= ""
		and specializationFilename ~= "" and fileExists(specializationFilename)
		and searchedSpecializations ~= ""
	then
		if g_specializationManager:getSpecializationByName(specializationName) == nil then
			g_specializationManager:addSpecialization(specializationName, specializationClassName, specializationFilename, nil);
			
			for vehicleName, vehicleType in pairs(g_vehicleTypeManager.types) do
				if vehicleType ~= nil and vehicleName ~= "locomotive" then
					local isDischargeable  = false;
					local hasWheels = false;
					local hasFillVolume = false;
					local hasFillUnit = false 
					local isBaler = false;
					local isConveyorBelt = false;
					for specName, specObject in pairs(vehicleType.specializationsByName) do
						--[[
						for _, searchedSpecialization in pairs(searchedSpecializations) do	
							if string.lower(name) == string.lower(searchedSpecialization) then
								local specializationObject = g_specializationManager:getSpecializationObjectByName(specializationName);
								
								if vehicleType.specializationsByName[specializationName] == nil then
									vehicleType.specializationsByName[specializationName] = specializationObject;
									
									table.insert(vehicleType.specializationNames, specializationName);
									table.insert(vehicleType.specializations, specializationObject);
									
									printDebug("Added Specialization '" .. specializationName .. "' succsessfully to vehicle type '" .. vehicleName .. "'.", 1, true);
								end;
							end;
						end;--]]
						if specName == "fillUnit"  then 
						--obviously, we need something to be blown away
							hasFillUnit = true;
						elseif specName == "dischargeable" then 
						-- by this we try to exclude vehicles that can be filled, but not emptied, because they use up the filling. This would not be typical for a vehicle which is targeted by this mod.
							isDischargeable = true;
						elseif specName == "wheels" then 
						--just to be sure that we dont end up with something stationary. Maybe this could be omitted. Would this mod be apliccable to a sledge? ;-)
							hasWheels = true;
						elseif specName == "baler" then 
						-- just to be sure we don't blow away the filling of a baler.
							isBaler = true; 
						elseif specName == "fillVolume" then 
						-- only if the fillVolume ist present we can assume that the load is susceptible to be blown away by the wind
							hasFillVolume = true;
						elseif specName == "conveyorBelt" then 
						-- only if the fillVolume ist present we can assume that the load is susceptible to be blown away by the wind
							isConveyorBelt = true;
						end; 
					end;
					if isDischargeable and hasFillUnit and hasWheels and hasFillVolume and not isBaler and not isConveyorBelt then
						vehicleType.specializationsByName["WindyLoads"] = WindyLoads;
						table.insert(vehicleType.specializationNames, "WindyLoads");
						table.insert(vehicleType.specializations, WindyLoads); 
						printDebug(string.format("[WINDYLOADS]: Successfuly registered WindyLoads specialization to vehicle of type %s", vehicleName), 1, true);
					else
						--print(string.format("Vehicle type %s could not be registered for WindyLoads specialization, because it has the follwoing detected specialization status: hasFillUnit = %s, hasFillVolume = %s, isDischargeable= %s, isBaler = %s, hasWheels = %s.", vehicleName ,tostring(hasFillUnit), tostring(hasFillVolume), tostring(isDischargeable), tostring(isBaler), tostring(hasWheels)));
					end; 
				end;
			end;
		end;
	else
		if specializationName == nil then
			printError("Missing specialization name! Skipping this specialization now!", false, false);
		elseif specializationClassName == nil then
			printError("Missing specialization class name! Skipping specialization '" .. specializationName .. "' now!", false, false);
		elseif specializationFilename == nil then
			printError("Missing specialization filename! Skipping specialization '" .. specializationName .. "' now!", false, false);
		elseif searchedSpecializations == nil then
			printError("Missing searched specialization names! Skipping specialization '" .. specializationName .. "' now!", false, false);
		end;
	end;
	
	specializationNumber = specializationNumber + 1;
end;

delete(modDesc);